uniform sampler2D 		uColorMap;
uniform sampler2D 		uNormalMap;
uniform sampler2D 		uParallaxMap;
uniform float			uNormalGain;
uniform float			uParallaxScale;
uniform float			uParallaxBias;

varying vec2			vTexCoord;
varying vec3			vEyeVector;	// These are all in tangent space
varying vec3			vLight0Vector;
varying vec3			vLight1Vector;

const float				kSpecExponent = 5.0;
const float				kDiffuse = 0.8;
const float				kSpecular = 0.1;


void Light(in vec3 lightVector, in vec3 normal, in vec4 lightColor, in vec3 eyeVector, in float specExponent, inout vec4 totalDiffuse, inout vec4 totalSpecular)
{
	lightVector = normalize(lightVector);
	vec3 reflection = normalize(-reflect(lightVector, normal));
	
	totalDiffuse += gl_FrontMaterial.diffuse * lightColor * max(dot(normal, lightVector), 0.0);
	totalSpecular += lightColor * pow(max(dot(reflection, eyeVector), 0.0), specExponent);
}


#define LIGHT(idx, vector) Light(vector, normal, gl_LightSource[idx].diffuse, eyeVector, kSpecExponent, diffuse, specular)


void main()
{
	vec3 eyeVector = normalize(vEyeVector);
	
	float parallaxMap = texture2D(uParallaxMap, vTexCoord).x;
	parallaxMap = parallaxMap * uParallaxScale + uParallaxBias;
	
	vec2 texCoord = vTexCoord - parallaxMap * eyeVector.xy * vec2(-1.0, 1.0);
	
	vec3 normal = texture2D(uNormalMap, texCoord).xyz - 0.5;
	normal = normalize(normal);
	vec4 colorMap = texture2D(uColorMap, texCoord);
	
	vec4 diffuse = vec4(0.0), specular = vec4(0);
	
#ifdef OO_LIGHT_0_FIX
	LIGHT(0, normalize(vLight0Vector));
#endif
	LIGHT(1, normalize(vLight1Vector));
	
	diffuse += gl_FrontMaterial.ambient * gl_LightSource[0].ambient;
	
	vec4 color = diffuse * colorMap + specular * kSpecular;
	color.a = 1.0;
	
	gl_FragColor = color;
}
